@@ -1,13 +1,26 @@
-#include <Rcpp.h>
 #include <fstream>
 #include <iostream>
 #include <sstream>
-using namespace Rcpp;
+#include <vector>
+#include <string>
+#include <map>
+#include <set>
 
 #include "readstat.h"
 #include "haven_types.h"
 #include "tagged_na.h"
 
+#include "cpp11/strings.hpp"
+#include "cpp11/doubles.hpp"
+#include "cpp11/integers.hpp"
+#include "cpp11/r_string.hpp"
+#include "cpp11/list.hpp"
+#include "cpp11/raws.hpp"
+#include "cpp11/sexp.hpp"
+
+#include "cpp11/protect.hpp"
+#include "cpp11/function.hpp"
+
 double haven_double_value_udm(readstat_value_t value, readstat_variable_t* var, bool user_na) {
   if (readstat_value_is_tagged_missing(value)) {
     return make_tagged_na(tolower(readstat_value_tag(value)));
@@ -42,7 +55,7 @@ public:
 
   void add(const char* value, std::string label) {
     if (values_i_.size() > 0 || values_d_.size() > 0)
-      stop("Can't add string to integer/double labelset");
+      cpp11::stop("Can't add string to integer/double labelset");
 
     values_s_.push_back(value);
     labels_.push_back(label);
@@ -50,7 +63,7 @@ public:
 
   void add(int value, std::string label) {
     if (values_d_.size() > 0 || values_s_.size() > 0)
-      stop("Can't add integer to string/double labelset");
+      cpp11::stop("Can't add integer to string/double labelset");
 
     values_i_.push_back(value);
     labels_.push_back(label);
@@ -58,7 +71,7 @@ public:
 
   void add(double value, std::string label) {
     if (values_i_.size() > 0 || values_s_.size() > 0)
-      stop("Can't add double to integer/string labelset");
+      cpp11::stop("Can't add double to integer/string labelset");
 
     values_d_.push_back(value);
     labels_.push_back(label);
@@ -68,40 +81,40 @@ public:
     return labels_.size();
   }
 
-  RObject labels() const {
-    RObject out;
+  cpp11::sexp labels() const {
+    cpp11::sexp out;
 
     if (values_i_.size() > 0) {
       int n = values_i_.size();
-      IntegerVector values(n);
-      CharacterVector labels(n);
+      cpp11::writable::integers values(n);
+      cpp11::writable::strings labels(n);
 
       for (int i = 0; i < n; ++i) {
         values[i] = values_i_[i];
-        labels[i] = Rf_mkCharCE(labels_[i].c_str(), CE_UTF8);
+        labels[i] = labels_[i].c_str();
       }
 
       values.attr("names") = labels;
       out = values;
     } else if (values_d_.size() > 0) {
       int n = values_d_.size();
-      NumericVector values(n);
-      CharacterVector labels(n);
+      cpp11::writable::doubles values(n);
+      cpp11::writable::strings labels(n);
 
       for (int i = 0; i < n; ++i) {
         values[i] = values_d_[i];
-        labels[i] = Rf_mkCharCE(labels_[i].c_str(), CE_UTF8);
+        labels[i] = labels_[i].c_str();
       }
 
       values.attr("names") = labels;
       out = values;
     } else {
       int n = values_s_.size();
-      CharacterVector values(n), labels(n);
+      cpp11::writable::strings values(n), labels(n);
 
       for (int i = 0; i < n; ++i) {
-        values[i] = Rf_mkCharCE(values_s_[i].c_str(), CE_UTF8);
-        labels[i] = Rf_mkCharCE(labels_[i].c_str(), CE_UTF8);
+        values[i] = values_s_[i].c_str();
+        labels[i] = labels_[i].c_str();
       }
 
       values.attr("names") = labels;
@@ -115,13 +128,12 @@ public:
 // DfReader ------------------------------------------------------------------
 
 class DfReader {
-  FileExt ext_;
   FileVendor vendor_;
 
   int nrows_, nrowsAlloc_;
   int ncols_;
-  List output_;
-  CharacterVector names_;
+  cpp11::writable::list output_;
+  cpp11::writable::strings names_;
   bool user_na_;
 
   std::vector<std::string> val_labels_;
@@ -132,8 +144,12 @@ class DfReader {
   std::set<std::string> colsSkip_;
 
 public:
-  DfReader(FileExt ext, bool user_na = false): ext_(ext), vendor_(extVendor(ext)), nrows_(0), ncols_(0), user_na_(user_na) {
-  }
+  DfReader(FileExt ext, bool user_na = false) :
+    vendor_(extVendor(ext)),
+    nrows_(0),
+    ncols_(0),
+    output_(static_cast<R_xlen_t>(0)),
+    user_na_(user_na) { }
 
   void skipCols(const std::vector<std::string>& cols) {
     std::set<std::string> cols_set(cols.begin(), cols.end());
@@ -155,15 +171,15 @@ public:
 
     ncols_ = var_count - colsSkip_.size();
 
-    output_ = List(ncols_);
-    names_ = CharacterVector(ncols_);
+    output_.resize(ncols_);
+    names_.resize(ncols_);
     val_labels_.resize(ncols_);
     var_types_.resize(ncols_);
   }
 
   void setMetadata(const char *file_label) {
     if (file_label != NULL && strcmp(file_label, "") != 0) {
-      output_.attr("label") = CharacterVector::create(Rf_mkCharCE(file_label, CE_UTF8));
+      output_.attr("label") = file_label;
     }
   }
 
@@ -183,27 +199,27 @@ public:
 
     int var_index = readstat_variable_get_index_after_skipping(variable);
 
-    names_[var_index] = Rf_mkCharCE(name, CE_UTF8);
+    names_[var_index] = name;
 
     switch(readstat_variable_get_type(variable)) {
     case READSTAT_TYPE_STRING_REF:
     case READSTAT_TYPE_STRING:
-      output_[var_index] = CharacterVector(nrowsAlloc_);
+      output_[var_index] = cpp11::writable::strings(nrowsAlloc_);
       break;
     case READSTAT_TYPE_INT8:
     case READSTAT_TYPE_INT16:
     case READSTAT_TYPE_INT32:
     case READSTAT_TYPE_FLOAT:
     case READSTAT_TYPE_DOUBLE:
-      output_[var_index] = NumericVector(nrowsAlloc_);
+      output_[var_index] = cpp11::writable::doubles(nrowsAlloc_);
       break;
     }
 
-    RObject col = output_[var_index];
+    cpp11::sexp col(output_[var_index]);
 
     const char* var_label = readstat_variable_get_label(variable);
     if (var_label != NULL && strcmp(var_label, "") != 0) {
-      col.attr("label") = CharacterVector::create(Rf_mkCharCE(var_label, CE_UTF8));
+      col.attr("label") = var_label;
     }
 
     if (val_labels != NULL)
@@ -219,11 +235,11 @@ public:
       col.attr("class") = "Date";
       break;
     case HAVEN_TIME:
-      col.attr("class") = CharacterVector::create("hms", "difftime");
+      col.attr("class") = {"hms", "difftime"};
       col.attr("units") = "secs";
       break;
     case HAVEN_DATETIME:
-      col.attr("class") = CharacterVector::create("POSIXct", "POSIXt");
+      col.attr("class") = {"POSIXct", "POSIXt"};
       col.attr("tzone") = "UTC";
       break;
     default:
@@ -237,16 +253,35 @@ public:
       case READSTAT_TYPE_STRING_REF:
       case READSTAT_TYPE_STRING:
       {
-        CharacterVector na_values(n_ranges);
+        cpp11::writable::strings na_values(R_xlen_t(0));
+        cpp11::writable::strings na_range(2);
+        bool has_range = false;
 
         for (int i = 0; i < n_ranges; ++i) {
-          readstat_value_t value = readstat_variable_get_missing_range_lo(variable, i);
-          const char* str_value = readstat_string_value(value);
-          na_values[0] = str_value == NULL ? NA_STRING : Rf_mkCharCE(str_value, CE_UTF8);
+          readstat_value_t
+            lo_value = readstat_variable_get_missing_range_lo(variable, i),
+            hi_value = readstat_variable_get_missing_range_hi(variable, i);
+
+          const char* lo = readstat_string_value(lo_value);
+          const char* hi = readstat_string_value(hi_value);
+
+          if (lo == hi) {
+            // single value
+            na_values.push_back(lo == NULL ? cpp11::r_string(NA_STRING) : cpp11::r_string(lo));
+          } else {
+            has_range = true;
+            // Can only ever be one range
+            na_range[0] = lo;
+            na_range[1] = hi;
+          }
         }
 
-        col.attr("na_values") = na_values;
-        col.attr("class") = CharacterVector::create("haven_labelled_spss", "haven_labelled", "vctrs_vctr", "character");
+        if (na_values.size() > 0)
+          col.attr("na_values") = na_values;
+        if (has_range)
+          col.attr("na_range") = na_range;
+
+        col.attr("class") = {"haven_labelled_spss", "haven_labelled", "vctrs_vctr", "character"};
         break;
       }
       case READSTAT_TYPE_INT8:
@@ -256,7 +291,7 @@ public:
       case READSTAT_TYPE_DOUBLE:
       {
         std::vector<double> na_values;
-        NumericVector na_range(2);
+        cpp11::writable::doubles na_range(2);
         bool has_range = false;
 
         for (int i = 0; i < n_ranges; ++i) {
@@ -280,11 +315,11 @@ public:
         }
 
         if (na_values.size() > 0)
-          col.attr("na_values") = Rcpp::wrap(na_values);
+          col.attr("na_values") = na_values;
         if (has_range)
           col.attr("na_range") = na_range;
 
-        col.attr("class") = CharacterVector::create("haven_labelled_spss", "haven_labelled", "vctrs_vctr", "double");
+        col.attr("class") = {"haven_labelled_spss", "haven_labelled", "vctrs_vctr", "double"};
       }
       }
     }
@@ -292,7 +327,7 @@ public:
 
     // Store original format as attribute
     if (var_format != NULL && strcmp(var_format, "") != 0) {
-      col.attr(formatAttribute(vendor_)) = Rf_ScalarString(Rf_mkCharCE(var_format, CE_UTF8));
+      col.attr(formatAttribute(vendor_)) = var_format;
     }
 
     // Store original display width as attribute if it differs from the default
@@ -318,10 +353,20 @@ public:
     case READSTAT_TYPE_STRING_REF:
     case READSTAT_TYPE_STRING:
     {
-      CharacterVector col = output_[var_index];
-
+      cpp11::writable::strings col(output_[var_index]);
       const char* str_value = readstat_string_value(value);
-      col[obs_index] = str_value == NULL ? NA_STRING : Rf_mkCharCE(str_value, CE_UTF8);
+
+      if (readstat_value_is_tagged_missing(value)) {
+        col[obs_index] = NA_STRING;
+      } else if (!user_na_ && readstat_value_is_defined_missing(value, variable)) {
+        col[obs_index] = NA_STRING;
+      } else if (readstat_value_is_system_missing(value)) {
+        col[obs_index] = NA_STRING;
+      } else if (str_value == NULL) {
+        col[obs_index] = cpp11::r_string("");
+      } else {
+        col[obs_index] = cpp11::r_string(str_value);
+      }
       break;
     }
     case READSTAT_TYPE_INT8:
@@ -330,7 +375,7 @@ public:
     case READSTAT_TYPE_FLOAT:
     case READSTAT_TYPE_DOUBLE:
     {
-      NumericVector col = output_[var_index];
+      cpp11::writable::doubles col(output_[var_index]);
       double val = haven_double_value_udm(value, variable, user_na_);
       col[obs_index] = adjustDatetimeToR(vendor_, var_type, val);
       break;
@@ -372,7 +417,7 @@ public:
     nrowsAlloc_ = n;
 
     for (int i = 0; i < ncols_; ++i) {
-      Shield<SEXP> copy(Rf_lengthgets(output_[i], n));
+      cpp11::sexp copy(Rf_lengthgets(output_[i], n));
       Rf_copyMostAttrib(output_[i], copy);
       output_[i] = copy;
     }
@@ -384,16 +429,16 @@ public:
     }
   }
 
-  List output(const std::string& name_repair) {
+  cpp11::list output(const std::string& name_repair) {
     if (nrows_ != nrowsAlloc_)
       resizeCols(nrows_);
 
     for (int i = 0; i < output_.size(); ++i) {
-      RObject col = output_[i];
+      cpp11::sexp col(output_[i]);
 
       if (hasLabel(i)) {
-        if (col.attr("class") == R_NilValue) {
-          col.attr("class") = CharacterVector::create("haven_labelled", "vctrs_vctr", Rf_type2char(TYPEOF(col)));
+        if (Rf_getAttrib(col, R_ClassSymbol) == R_NilValue) {
+          col.attr("class") = {"haven_labelled", "vctrs_vctr", Rf_type2char(TYPEOF(col))};
         }
         col.attr("labels") = label_sets_[val_labels_[i]].labels();
       }
@@ -401,9 +446,10 @@ public:
 
     int nNotes = notes_.size();
     if (nNotes > 0) {
-      CharacterVector notes(nNotes);
+      cpp11::writable::strings notes(nNotes);
+
       for (int i = 0; i < nNotes; ++i) {
-        notes[i] = Rf_mkCharCE(notes_[i].c_str(), CE_UTF8);
+        notes[i] = notes_[i].c_str();
       }
 
       output_.attr("notes") = notes_;
@@ -411,10 +457,10 @@ public:
 
     output_.attr("names") = names_;
 
-    static Function as_tibble("as_tibble", Environment::namespace_env("tibble"));
-    return as_tibble(output_, Named(".rows", nrows_), Named(".name_repair", name_repair));
+    static cpp11::function as_tibble = cpp11::package("tibble")["as_tibble"];
+    using namespace cpp11::literals;
+    return SEXP(as_tibble(output_, ".rows"_nm = nrows_, ".name_repair"_nm = name_repair));
   }
-
 };
 
 int dfreader_metadata(readstat_metadata_t *metadata, void *ctx) {
@@ -439,7 +485,7 @@ int dfreader_value(int obs_index, readstat_variable_t *variable,
                    readstat_value_t value, void *ctx) {
   // Check for user interrupts every 10,000 rows or cols
   if ((obs_index + 1) % 10000 == 0 || (variable->index + 1) % 10000 == 0)
-    checkUserInterrupt();
+    cpp11::check_user_interrupt();
 
   ((DfReader*) ctx)->setValue(obs_index, variable, value);
   return 0;
@@ -451,7 +497,7 @@ int dfreader_value_label(const char *val_labels, readstat_value_t value,
 }
 
 void print_error(const char* error_message, void* ctx) {
-  Rcout << error_message << "\n";
+  Rprintf("%s\n", error_message);
 }
 
 
@@ -498,8 +544,8 @@ class DfReaderInputFile : public DfReaderInputStream<std::ifstream> {
   std::string filename_;
 
 public:
-  DfReaderInputFile(Rcpp::List spec, std::string encoding = "") {
-    CharacterVector path(spec[0]);
+  DfReaderInputFile(cpp11::list spec, std::string encoding = "") {
+    cpp11::strings path(spec[0]);
     filename_ = std::string(Rf_translateChar(path[0]));
     this->encoding = encoding;
   }
@@ -521,8 +567,8 @@ public:
 
 class DfReaderInputRaw : public DfReaderInputStream<std::istringstream> {
 public:
-  DfReaderInputRaw(Rcpp::List spec, std::string encoding = "") {
-    Rcpp::RawVector raw_data(spec[0]);
+  DfReaderInputRaw(cpp11::list spec, std::string encoding = "") {
+    cpp11::raws raw_data(spec[0]);
     std::string string_data((char*) RAW(raw_data), Rf_length(raw_data));
     file_.str(string_data);
     this->encoding = encoding;
@@ -607,17 +653,18 @@ void haven_parse(readstat_parser_t* parser, DfReaderInput& builder_input, DfRead
   if (result != READSTAT_OK) {
     std::string source = builder_input.source();
     readstat_parser_free(parser);
-    stop("Failed to parse %s: %s.", source, readstat_error_message(result));
+    std::string msg(readstat_error_message(result));
+    cpp11::stop("Failed to parse %s: %s.", source.c_str(), msg.c_str());
   }
 }
 
 template<FileExt ext, typename InputClass>
-List df_parse(const List& spec, const std::vector<std::string>& cols_skip,
+cpp11::list df_parse(cpp11::list spec, const std::vector<std::string>& cols_skip,
               const long& n_max = -1, const long& rows_skip = 0,
               const std::string& encoding = "",
               const bool& user_na = false,
               const std::string& name_repair = "check_unique",
-              const List& catalog_spec = List(),
+              cpp11::list catalog_spec = cpp11::writable::list(R_xlen_t(0)),
               const std::string& catalog_encoding = ""
               ) {
   DfReader builder(ext, user_na);
@@ -645,54 +692,54 @@ List df_parse(const List& spec, const std::vector<std::string>& cols_skip,
 
 // # nocov start
 
-// [[Rcpp::export]]
-List df_parse_sas_file(Rcpp::List spec_b7dat, Rcpp::List spec_b7cat,
+[[cpp11::register]]
+cpp11::list df_parse_sas_file(cpp11::list spec_b7dat, cpp11::list spec_b7cat,
                        std::string encoding, std::string catalog_encoding,
                        std::vector<std::string> cols_skip, long n_max, long rows_skip,
                        std::string name_repair) {
   return df_parse<HAVEN_SAS7BDAT, DfReaderInputFile>(spec_b7dat, cols_skip, n_max, rows_skip, encoding, false, name_repair, spec_b7cat, catalog_encoding);
 }
-// [[Rcpp::export]]
-List df_parse_sas_raw(Rcpp::List spec_b7dat, Rcpp::List spec_b7cat,
+[[cpp11::register]]
+cpp11::list df_parse_sas_raw(cpp11::list spec_b7dat, cpp11::list spec_b7cat,
                       std::string encoding, std::string catalog_encoding,
                       std::vector<std::string> cols_skip, long n_max, long rows_skip,
                       std::string name_repair) {
   return df_parse<HAVEN_SAS7BDAT, DfReaderInputRaw>(spec_b7dat, cols_skip, n_max, rows_skip, encoding, false, name_repair, spec_b7cat, catalog_encoding);
 }
 
-// [[Rcpp::export]]
-List df_parse_xpt_file(Rcpp::List spec, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
+[[cpp11::register]]
+cpp11::list df_parse_xpt_file(cpp11::list spec, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
   return df_parse<HAVEN_XPT, DfReaderInputFile>(spec, cols_skip, n_max, rows_skip, "", false, name_repair);
 }
-// [[Rcpp::export]]
-List df_parse_xpt_raw(Rcpp::List spec, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
+[[cpp11::register]]
+cpp11::list df_parse_xpt_raw(cpp11::list spec, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
   return df_parse<HAVEN_XPT, DfReaderInputRaw>(spec, cols_skip, n_max, rows_skip, "", false, name_repair);
 }
 
-// [[Rcpp::export]]
-List df_parse_dta_file(Rcpp::List spec, std::string encoding, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
+[[cpp11::register]]
+cpp11::list df_parse_dta_file(cpp11::list spec, std::string encoding, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
   return df_parse<HAVEN_DTA, DfReaderInputFile>(spec, cols_skip, n_max, rows_skip, encoding, false, name_repair);
 }
-// [[Rcpp::export]]
-List df_parse_dta_raw(Rcpp::List spec, std::string encoding, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
+[[cpp11::register]]
+cpp11::list df_parse_dta_raw(cpp11::list spec, std::string encoding, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
   return df_parse<HAVEN_DTA, DfReaderInputRaw>(spec, cols_skip, n_max, rows_skip, encoding, false, name_repair);
 }
 
-// [[Rcpp::export]]
-List df_parse_sav_file(Rcpp::List spec, std::string encoding, bool user_na, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
+[[cpp11::register]]
+cpp11::list df_parse_sav_file(cpp11::list spec, std::string encoding, bool user_na, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
   return df_parse<HAVEN_SAV, DfReaderInputFile>(spec, cols_skip, n_max, rows_skip, encoding, user_na, name_repair);
 }
-// [[Rcpp::export]]
-List df_parse_sav_raw(Rcpp::List spec, std::string encoding, bool user_na, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
+[[cpp11::register]]
+cpp11::list df_parse_sav_raw(cpp11::list spec, std::string encoding, bool user_na, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
   return df_parse<HAVEN_SAV, DfReaderInputRaw>(spec, cols_skip, n_max, rows_skip, encoding, user_na, name_repair);
 }
 
-// [[Rcpp::export]]
-List df_parse_por_file(Rcpp::List spec, std::string encoding, bool user_na, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
+[[cpp11::register]]
+cpp11::list df_parse_por_file(cpp11::list spec, std::string encoding, bool user_na, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
   return df_parse<HAVEN_POR, DfReaderInputFile>(spec, cols_skip, n_max, rows_skip, encoding, user_na, name_repair);
 }
-// [[Rcpp::export]]
-List df_parse_por_raw(Rcpp::List spec, std::string encoding, bool user_na, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
+[[cpp11::register]]
+cpp11::list df_parse_por_raw(cpp11::list spec, std::string encoding, bool user_na, std::vector<std::string> cols_skip, long n_max, long rows_skip, std::string name_repair) {
   return df_parse<HAVEN_POR, DfReaderInputRaw>(spec, cols_skip, n_max, rows_skip, encoding, user_na, name_repair);
 }
 
